from datetime import datetime
from enum import IntEnum
from typing import Any


class ProbeBatteryLevel(IntEnum):
    """Map battery status to corresponding level in percentage.

    The calculated percentage is only an approximation.
    """

    NORMAL = 100
    FAIR = 50
    POOR = 10


class Sensor:
    type: str | None = None
    depth: int | None = None
    value: float | None = None
    unit: str | None = None
    timestamp: datetime | None = None
    soil: str | None = None

    def __init__(self, id: str):
        self.id = id

    def __repr__(self) -> str:
        return self.id

    def __str__(self) -> str:
        parts = [
            str(self.timestamp),
            f"{self.type:14}",
            f"{self.value:2.0f} {self.unit:2}",
        ]
        if self.depth is not None:
            parts.append(f"@ {self.depth} cm")
        if self.soil is not None:
            parts.append(f"in {self.soil}")
        return " ".join(parts)

    def update(self, reading: dict[str, Any], soil: str | None = None) -> None:
        self.type = reading["type"]
        self.depth = int(reading["depth"]) if "depth" in reading else None
        self.value = float(reading["value"])
        self.unit = reading["unit"]
        if reading["timestamp"] is not None:
            self.timestamp = datetime.fromisoformat(reading["timestamp"])
        self.soil = soil


class Probe:
    name: str | None = None
    sku: str | None = None
    state: str | None = None
    location_id: int | None = None
    location: str | None = None
    latitude: float | None = None
    longitude: float | None = None

    def __init__(self, serial: str):
        self.serial = serial
        self.__sensors: dict[str, Sensor] = {}

    def __find_soil(
        self,
        soils: dict[str, str],
        profile: list[dict[str, Any]],
        reading: dict[str, Any],
    ) -> str | None:
        """Find the soil at depth based on the soil profile

        We can safely assume profile is sorted on depth.
        """
        soil = None
        if reading["type"] == "MOISTURE" and "depth" in reading:
            for layer in profile:
                if layer["depth"] > reading["depth"]:
                    break
                soil = soils.get(layer["soil"])
        return soil

    def __update_sensor(
        self, id: str, reading: dict[str, Any], soil: str | None = None
    ) -> None:
        if id not in self.__sensors:
            self.__sensors[id] = Sensor(id)
        self.__sensors[id].update(reading, soil)

    def update(
        self, soils: dict[str, str], locations: dict[int, str], probe: dict[str, Any]
    ) -> None:
        self.id = probe["id"]
        self.name = probe["name"]
        self.sku = probe["sku"]
        self.state = probe["state"]

        self.latitude = probe.get("latitude")
        self.longitude = probe.get("longitude")

        if "location_id" in probe:
            self.location_id = int(probe["location_id"])
            self.location = locations.get(self.location_id)
        else:
            self.location_id = None
            self.location = None

        timestamp = probe["status"].get("last_update")
        if timestamp is not None:
            self.timestamp = datetime.fromisoformat(timestamp)

        if "last_readings" in probe["status"]:
            for reading in probe["status"]["last_readings"]:
                id = f"{probe['serial']}-{reading['type']}"
                if "depth" in reading and reading["depth"] > 0:
                    id += "-" + str(reading["depth"])
                soil = self.__find_soil(soils, probe["soil_profile"], reading)
                self.__update_sensor(id, reading, soil)

        if "battery" in probe["status"]:
            self.__update_sensor(
                f"{probe['serial']}-BATTERY",
                {
                    "type": "BATTERY",
                    "value": ProbeBatteryLevel[probe["status"].get("battery")].value,
                    "unit": "%",
                    "timestamp": timestamp,
                },
            )

        if "rssi" in probe["status"]:
            self.__update_sensor(
                f"{probe['serial']}-RSSI",
                {
                    "type": "RSSI",
                    "value": probe["status"]["rssi"],
                    "unit": "dBm",
                    "timestamp": timestamp,
                },
            )

    def sensors(self) -> list[Sensor]:
        """Get the list of sensors of a probe"""
        return list(self.__sensors.values())

    def __str__(self) -> str:
        parts = [f"-{self.serial}-"]
        if self.name != self.serial:
            parts.append(f"({self.name})")
        if self.location is not None:
            parts.append(f"@{self.location}")
        return " ".join(parts)

    def __repr__(self) -> str:
        return f"{self.serial}"



import asyncio
import time
from datetime import datetime

from sensoterra.customerapi import CustomerApi, InvalidAuth, Timeout


async def main():
    if True:
        api = CustomerApi("mark.ruys@sensoterra.com", "Kubus1966!")
        print(api.get_version())
    else:
        api = CustomerApi("cus577@sensoterra.com", "jcmFE6Gfpn7KxTs2aHk6")
        api.api_base_url = "http://localhost:8001/api"
    api.set_language("nl")

    if True:
        token = await api.get_token(
            "Home Assistant", "READONLY", datetime(2038, 1, 1, 0, 0)
        )
        api = CustomerApi()
        api.set_language("nl")
        api.set_token(token)

    while True:
        try:
            probes = await api.poll()

            for probe in probes:
                if probe.serial != "24000092104":
                    continue
                print(probe)
                for sensor in probe.sensors():
                    print(sensor)
                print()
        except Timeout as exp:
            print("Timeout")
        except InvalidAuth as exp:
            print(exp)

        time.sleep(10)
        print("-" * 70)


asyncio.run(main())



import asyncio
from datetime import datetime
from importlib import metadata
from typing import Any

from aiohttp import ClientSession

from .probe import Probe

VERSION = metadata.version(__package__)


class InvalidAuth(Exception):
    def __init__(self, message: str):
        self.message = message
        super().__init__(self.message)


class Timeout(Exception):
    pass


class CustomerApi:

    API_TIMEOUT_NORMAL = 3
    API_TIMEOUT_SLOW = 10
    DEFAULT_LANGUAGE = "en"

    api_base_url = "https://monitor.sensoterra.com/api/v3"

    def __init__(self, email: str | None = None, password: str | None = None):
        self.__email = email
        self.__password = password
        self.__headers = {
            "User-Agent": "Sensoterra Python API/{}".format(self.get_version()),
            "language": self.DEFAULT_LANGUAGE,
        }
        self.__probes: dict[str, Probe] = {}
        self.__api_token: str | None = None
        self.__include_shared_data = True

    async def __auth(self, session: ClientSession) -> None:
        """Authenticate using either email and password, or an API token"""
        if self.__api_token is None:
            async with session.post(
                f"{self.api_base_url}/customer/auth",
                json={"email": self.__email, "password": self.__password},
                timeout=self.API_TIMEOUT_NORMAL,
            ) as resp:
                if resp.status >= 400 and resp.status <= 499:
                    json = await resp.json()
                    raise InvalidAuth(json["message"])
                resp.raise_for_status()
                json = await resp.json()
                api_key = json["api_key"]
        else:
            api_key = self.__api_token

        self.__headers["api_key"] = api_key

    async def __request(
        self,
        session: ClientSession,
        method: str,
        command: str,
        params: dict[str, str] | None = None,
        json: dict[str, str] | None = None,
    ) -> Any:
        """Wrapper to requests.request

        Returns:
        None: in case a http status 429 is received, or when poll too often
        Response: request response object
        """

        for i in range(2):
            if "api_key" not in self.__headers:
                await self.__auth(session)
            async with session.request(
                method,
                f"{self.api_base_url}/{command}",
                headers=self.__headers,
                params=params,
                json=json,
                timeout=self.API_TIMEOUT_SLOW,
            ) as resp:
                if resp.status == 403:  # Forbidden
                    await self.__auth(session)
                    json = None
                elif resp.status == 401:  # Unauthorized
                    json = await resp.json()
                    msg = json["message"] if json is not None else "Unauthorized"
                    raise InvalidAuth(msg)
                else:
                    resp.raise_for_status()
                    json = await resp.json()
                    break

        return json

    async def __get_locations(self, session: ClientSession) -> dict[int, str]:
        """Get all locations"""
        params = {"include_shared_data": "YES" if self.__include_shared_data else "NO"}

        locations = await self.__request(session, "GET", "location", params=params)

        return {location["id"]: location["name"] for location in locations}

    async def __get_soils(self, session: ClientSession) -> dict[str, str]:
        """Get all soil types"""

        parameters = await self.__request(session, "GET", "parameter")

        return {
            parameter["key"]: parameter["name"]
            for parameter in parameters
            if parameter["type"] == "SOIL"
        }

    async def __get_probes(self, session: ClientSession) -> dict[str, str]:
        """Get all probes"""
        params = {"include_shared_data": "YES" if self.__include_shared_data else "NO"}

        probes: dict[str, str] = await self.__request(
            session, "GET", "probe", params=params
        )
        return probes

    def get_version(self) -> str:
        return VERSION

    def set_language(self, language: str) -> None:
        """Set langage like for soil names

        Examples: es, en, en-us, nl
        """
        self.__headers["language"] = language

    def set_token(self, token: str) -> None:
        self.__api_token = token

    def get_include_shared_data(self) -> bool:
        return self.__include_shared_data

    def set_include_shared_data(self, include_shared_data: bool) -> None:
        self.__include_shared_data = include_shared_data

    async def get_token(self, tag: str, scope: str, expiration: datetime) -> str:
        async with ClientSession() as session:
            data = {
                "tag": tag,
                "scope": scope,
                "expiration": expiration.astimezone().isoformat(),
            }
            json = await self.__request(session, "POST", "token", json=data)
        return str(json["token"])

    async def poll(self) -> list[Probe]:
        """Update the list of probes"""
        try:
            async with ClientSession() as session:
                await self.__auth(session)
                async with asyncio.TaskGroup() as tg:
                    task_1 = tg.create_task(self.__get_probes(session))
                    task_2 = tg.create_task(self.__get_soils(session))
                    task_3 = tg.create_task(self.__get_locations(session))
                probes: list[dict[str, Any]] = task_1.result()  # type: ignore
                soils = task_2.result()
                locations = task_3.result()
        except ExceptionGroup as exp:
            for inner in exp.exceptions:
                if isinstance(inner, InvalidAuth):
                    raise inner
                if isinstance(inner, TimeoutError):
                    raise Timeout
            raise exp

        for probe in probes:
            if probe["serial"] not in self.__probes:
                self.__probes[probe["serial"]] = Probe(probe["serial"])
            self.__probes[probe["serial"]].update(soils, locations, probe)

        return list(self.__probes.values())



import os
import time

import pytest

from sensoterra import customerapi


@pytest.mark.asyncio
async def test_customer_api() -> None:
    email = os.environ.get("EMAIL")
    password = os.environ.get("PASSWORD")
    api_url = os.environ.get("CUSTOMER_API_URL")
    api_version = os.environ.get("CUSTOMER_API_VERSION")

    assert email is not None
    assert password is not None

    api = customerapi.CustomerApi(email, password)
    if api_url is not None:
        api.api_base_url = api_url
    if api_version is not None:
        api.api_version = api_version

    probes = await api.poll()

    # We should have probes
    assert len(probes) > 0

    # Some probes should have sensors
    assert any([len(probe.sensors()) > 0 for probe in probes])
